unit Configuration;

{$mode objfpc}{$H+}

interface

uses
{$IFDEF Win32}
  Windows,
{$ENDIF}
  Classes, SysUtils, IniFiles, Main_Const;
  
type

  { TConfiguration }

  TConfiguration = class
  private
    ConfigFilePath : string;
  public
   MainDir : string;
   ResW,
   ResH,
   ResB : Word;
   FullScreen : Boolean;
   Map,
   BackGround : Integer;
   TourTime,
   GameTime : Integer;
   constructor Create;
   destructor Destroy; override;
   procedure ReadFromFile;
   procedure Save;
  end;
  
var
  MainConfig : TConfiguration;

implementation

const

  SectionSystem = 'System';
  SectionVideo = 'Video';
  SectionSound = 'Sound';
  SectionGame = 'Game';

  IdentMainDir = 'MainDir';
  IdentResW = 'ResW';
  IdentResH = 'ResH';
  IdentResB = 'ResB';
  IdentFullScreen = 'FullScreen';
  IdentMap = 'Map';
  IdentBackGround = 'Background';
  IdentTourTime = 'TourTime';
  IdentGameTime = 'GameTime';

{ TConfiguration }

constructor TConfiguration.Create;
begin
{$ifdef win32}
  ConfigFilePath := ExtractFilePath(Application.EXEName) + SHORT_TITLE + '.ini';
{$endif}
{$ifdef Unix}
  ConfigFilePath := GetAppConfigFile(False);
{$endif}

  ReadFromFile;
end;

destructor TConfiguration.Destroy;
begin
  Save;
  
  inherited Destroy;
end;

procedure TConfiguration.ReadFromFile;
var
  _File : TIniFile;
  _Dir : string;
begin
  _File := TIniFile.Create(ConfigFilePath);
  try
{$ifdef Win32}
    _Dir := ExtractFilePath(Application.EXEName);
{$else}
    _Dir := GetAppConfigDir(False);
{$endif}
    //odczytywanie konfiguracji
    MainDir := _File.ReadString(SectionSystem, IdentMainDir, _Dir);
    ResW := _File.ReadInteger(SectionVideo, IdentResW, DEF_RES_W);
    ResH := _File.ReadInteger(SectionVideo, IdentResH, DEF_RES_H);
    ResB := _File.ReadInteger(SectionVideo, IdentResB, DEF_RES_B);
    FullScreen := _File.ReadBool(SectionVideo, IdentFullScreen, True);
    Map := _File.ReadInteger(SectionGame, IdentMap, 0);
    BackGround := _File.ReadInteger(SectionGame, IdentBackGround, 0);
    TourTime := _File.ReadInteger(SectionGame, IdentTourTime, 0);
    GameTime := _File.ReadInteger(SectionGame, IdentGameTime, 0);
  finally
    _File.Free;
  end;
end;

procedure TConfiguration.Save;
var
  _File: TIniFile;
begin
  _File := TIniFile.Create(ConfigFilePath);
  try
    //zapisywanie konfiguracji
    _File.WriteString(SectionSystem, IdentMainDir, MainDir);
    _File.WriteInteger(SectionVideo, IdentResW, ResW);
    _File.WriteInteger(SectionVideo, IdentResH, ResH);
    _File.WriteInteger(SectionVideo, IdentResB, ResB);
    _File.WriteBool(SectionVideo, IdentFullScreen, FullScreen);
    _File.WriteInteger(SectionGame, IdentMap, Map);
    _File.WriteInteger(SectionGame, IdentBackGround, BackGround);
    _File.WriteInteger(SectionGame, IdentTourTime, TourTime);
    _File.WriteInteger(SectionGame, IdentGameTime, GameTime);
  finally
    _File.Free;
  end;
end;

initialization
  MainConfig := TConfiguration.Create;
  
finalization
  FreeAndNil(MainConfig);

end.

