unit frmMain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, Buttons,
  ComCtrls, Game, ExtCtrls, StdCtrls, Configuration, Main_Const, frmCreateTeam,
  TeamManager;

type

  { Tfrm_Main }

  Tfrm_Main = class(TForm)
    btn_Start: TButton;
    btn_Exit: TButton;
    btn_Add: TButton;
    btn_Remove: TButton;
    btn_CreateTeam: TButton;
    cb_FullScreen: TCheckBox;
    cob_Background: TComboBox;
    cob_Ground: TComboBox;
    cob_GameTime: TComboBox;
    cob_TourTime: TComboBox;
    gb_Teams: TGroupBox;
    gb_Map: TGroupBox;
    Image1: TImage;
    lbl_Avalaibe: TLabel;
    lbl_InGame: TLabel;
    lbl_TourTime: TLabel;
    lbl_GameTime: TLabel;
    lbl_Ground: TLabel;
    lbl_Background: TLabel;
    lb_TeamsInGame: TListBox;
    lb_Teams: TListBox;
    mem_About: TMemo;
    PageControl1: TPageControl;
    rg_Resolution: TRadioGroup;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    procedure FormCreate(Sender: TObject);
    procedure Image1Click(Sender: TObject);
    procedure btn_AddClick(Sender: TObject);
    procedure btn_CreateTeamClick(Sender: TObject);
    procedure btn_ExitClick(Sender: TObject);
    procedure btn_RemoveClick(Sender: TObject);
    procedure btn_StartClick(Sender: TObject);
    procedure cob_BackgroundChange(Sender: TObject);
    procedure cob_GroundChange(Sender: TObject);
  private
    procedure SetConfiguration;
  public
    RobakiGame : TGame;
    procedure AddTeam(const ATeam : TTeam);
  end; 

var
  frm_Main: Tfrm_Main;

implementation

{ Tfrm_Main }

procedure Tfrm_Main.btn_StartClick(Sender: TObject);
var
  i: Integer;
begin
  //czy sa druzyny w grze?
  if lb_TeamsInGame.Items.Count = 0 then
  begin
    ShowMessage('Dodaj conajmniej jedna druzyne do gry');
    Exit;
  end;
  ;
  //zapisanie konfiguracji
  SetConfiguration;
  
  //uruchomienie gry
  RobakiGame := TGame.Create;
  with RobakiGame do
  begin
    case rg_Resolution.ItemIndex of
      0:
      begin
        Scr_Width := 800;
        Scr_Height := 600;
      end;
      1:
      begin
        Scr_Width := 960;
        Scr_Height := 600;
      end;
    end;
    Scr_BPP := DEF_RES_B;
    FullScreen := cb_FullScreen.Checked;
  end;
  
  for i := 0 to lb_TeamsInGame.Items.Count - 1 do
  begin
    TeamMngrInGame.AddTeam(TTeam(lb_TeamsInGame.Items.Objects[i]));
  end;
  
  //start
  RobakiGame.TourTime := cob_TourTime.ItemIndex + 1;
  case cob_GameTime.ItemIndex of
  0: RobakiGame.GameTime := 10;
  1: RobakiGame.GameTime := 15;
  2: RobakiGame.GameTime := 25;
  3: RobakiGame.GameTime := 40;
  4: RobakiGame.GameTime := 60;
  end;
  RobakiGame.FPS := 30;
  RobakiGame.Start;
end;

procedure Tfrm_Main.cob_BackgroundChange(Sender: TObject);
begin
  SetConfiguration;
end;

procedure Tfrm_Main.cob_GroundChange(Sender: TObject);
begin
  SetConfiguration;
end;

procedure Tfrm_Main.SetConfiguration;
var
  _Width, _Height : Word;
begin
  MainConfig.FullScreen := cb_FullScreen.Checked;
  case rg_Resolution.ItemIndex of
      0: _Width := 800;
      1: _Width := 960;
  end;
  
  _Height := 600;
  
  MainConfig.ResW := _Width;
  MainConfig.ResH := _Height;
  MainConfig.ResB := DEF_RES_B;
  
  MainConfig.BackGround := cob_BackGround.ItemIndex;
  MainConfig.Map := cob_Ground.ItemIndex;
  
  MainConfig.TourTime := cob_TourTime.ItemIndex;
  MainConfig.GameTime := cob_GameTime.ItemIndex;
  
  //MainConfig.Save;
end;

procedure Tfrm_Main.AddTeam(const ATeam: TTeam);
begin
  //dodanie druzyny
  lb_Teams.Items.AddObject(ATeam.Name, ATeam);
end;

procedure Tfrm_Main.btn_ExitClick(Sender: TObject);
begin
  //zapisanie konfiguracji
  SetConfiguration;
  
  //wyjscie
  Application.Terminate;
end;

procedure Tfrm_Main.btn_RemoveClick(Sender: TObject);
var
  _tempIndex : Integer;
begin
  _tempIndex := lb_TeamsInGame.ItemIndex;
  if _tempIndex = -1 then
    Exit;
  lb_Teams.Items.AddObject(lb_TeamsInGame.Items[_tempIndex], lb_TeamsInGame.Items.Objects[_tempIndex]);
  lb_TeamsInGame.Items.Delete(_tempIndex);
end;

procedure Tfrm_Main.FormCreate(Sender: TObject);
begin
  cb_FullScreen.Checked := MainConfig.FullScreen;
  if MainConfig.ResW = 800 then
    rg_Resolution.ItemIndex := 0
  else
    rg_Resolution.ItemIndex := 1;
    
  //Zaladowanie zasobow
  cob_BackGround.Items.LoadFromFile(MainConfig.MainDir + '/tla/tla');
  cob_BackGround.ItemIndex := MainConfig.BackGround;
  cob_Ground.Items.LoadFromFile(MainConfig.MainDir + '/mapy/mapy');
  cob_Ground.ItemIndex := MainConfig.Map;
  cob_TourTime.ItemIndex := MainConfig.TourTime;
  cob_GameTime.ItemIndex := MainConfig.GameTime;
end;

procedure Tfrm_Main.Image1Click(Sender: TObject);
begin

end;

procedure Tfrm_Main.btn_AddClick(Sender: TObject);
var
  _tempIndex : Integer;
begin
  _tempIndex := lb_Teams.ItemIndex;
  if _tempIndex = -1 then
    Exit;
  lb_TeamsInGame.Items.AddObject(lb_Teams.Items[_tempIndex], lb_Teams.Items.Objects[_tempIndex]);
  lb_Teams.Items.Delete(_tempIndex);
end;

procedure Tfrm_Main.btn_CreateTeamClick(Sender: TObject);
begin
  frm_CreateTeam.Show;
end;

initialization
  {$I frmmain.lrs}

end.

