unit Game;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Timer, sdl, Main_Const,
  Configuration, MiniEngine;
  
type

  { TGame }

  TGame = class
  private
    FQuit : Boolean;
    FTimer : TTimer;
    Screen : PSDL_Surface;
    Event : TSDL_Event;
    MousePos,
    MousePercent : TPoint;
    FEngine : TMiniEngine;

    //glowna petla
    procedure Main;
  public
    TourTime,
    GameTime : Integer;
    Scr_Width,
    Scr_Height,
    Scr_BPP : Integer;
    FullScreen : Boolean;
    FPS : Byte;
    constructor Create;
    //rozpoczecie gry
    procedure Start;
  end;

implementation

{ TGame }

constructor TGame.Create;
begin
  FTimer := TTimer.Create;
  FQuit := False;
  Scr_Width := DEF_RES_W;
  Scr_Height := DEF_RES_H;
  Scr_BPP := DEF_RES_B;
  FPS := 30;
  FullScreen := False;
  
end;

procedure TGame.Start;
var
  _ScreenMode : Integer;
begin
  //inicjalizacja SDL
  if SDL_Init(SDL_INIT_EVERYTHING) = -1 then
  begin
  //blad
  end;
  
  if FullScreen then
    _ScreenMode := SDL_SWSURFACE or SDL_DOUBLEBUF or SDL_FULLSCREEN
  else
    _ScreenMode := SDL_SWSURFACE or SDL_DOUBLEBUF;
    
  Screen := SDL_SetVideoMode(Scr_Width, Scr_Height, Scr_BPP, _ScreenMode);
  if Screen = nil then
  begin
  //blad
  end;
  
  SDL_WM_SetCaption(GAME_TITLE, nil);
  
  //ustawienie FPS
  FTimer.Fps := FPS;
  
  //tworzenie silnika
  FEngine := TMiniEngine.Create(Screen);
  FEngine.OP := @MousePos;
  FEngine.OPP := @MousePercent;
  FEngine.Timer := @FTimer;
  FEngine.TourTime := TourTime;
  FEngine.GameTime := GameTime;
  FEngine.LoadGraphics;
  

  //uruchomienie glownej petli
  Main;
  
  //po zakonczeniu glownej petli
  FreeAndNil(FEngine);
  
  //zwolnienie SDL
  SDL_Quit;
end;

procedure TGame.Main;
begin
  while not FQuit do
  begin
    FTimer.Start;
    //glowna petla
    
    //przetwarzanie zdarzen
    while SDL_PollEvent(@Event) > 0 do
    begin
      //wyjscie
      if Event.type_ = SDL_QUITEV then
        FQuit := True;
        
      //nacisniecie przycisku
      if Event.type_ = SDL_KEYDOWN then
      begin
        case Event.key.keysym.sym of
          SDLK_ESCAPE:
            //wyjscie
            FQuit := True;
        end;
        
        FEngine.KeyDown(Event.key.keysym.sym);
      end;
      
      //zwolnienie przycisku
      if Event.type_ = SDL_KEYUP then
      begin
        FEngine.KeyUp(Event.key.keysym.sym);
      end;
      
      //ruch mysza
      if Event.type_ = SDL_MOUSEMOTION then
      begin
        MousePos.x := Event.button.x;
        MousePos.y := Event.button.y;
        MousePercent.x := Round(MousePos.x / Screen^.w * 1000);
        MousePercent.y := Round(MousePos.y / Screen^.h * 1000);
        
        FEngine.MouseMove;
      end;
      
      //klikniecie mysza
      if Event.type_ = SDL_MOUSEBUTTONDOWN then
      begin
        if Event.button.button = SDL_BUTTON_LEFT then
          FEngine.MouseLClick;
        if Event.button.button = SDL_BUTTON_RIGHT then
          FEngine.MouseRClick;
      end;
      
      {if Event.type_ = SDL_MOUSEBUTTONUP then
      begin
        if Event.button.button = SDL_BUTTON_LEFT then
          FEngine.MouseLUp;
        if Event.button.button = SDL_BUTTON_RIGHT then
          FEngine.MouseRUp;
      end;                }
    end;
    
    //silnik
    FEngine.MoveAll;
    FEngine.DrawAll;
    
    //wyswietlenie na ekranie
    SDL_Flip(Screen);
    
    //koniec glownej petli
    FTimer.Wait;
  end;
end;

end.

