unit GroundImage;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sdl, sdl_image;
  
type

  { TGroundImage }

  TGroundImage = class
  private
    FMousePercentPointer : PPoint;
    FScreen : PSDL_Surface;
    FColorKey : UInt32;
  public
    Image : PSDL_Surface;

    constructor Create(AScreen : PSDL_Surface; AMousePercent : PPoint);
    destructor Destroy;
    procedure LoadImage(AFile : string; AAlpha : Boolean = False);
    procedure Draw;
  end;

implementation

{ TGroundImage }

constructor TGroundImage.Create(AScreen: PSDL_Surface; AMousePercent: PPoint);
begin
  FMousePercentPointer := AMousePercent;
  FScreen := AScreen;
end;

destructor TGroundImage.Destroy;
begin
  SDL_FreeSurface(Image);
  inherited;
end;

procedure TGroundImage.LoadImage(AFile: string; AAlpha : Boolean = False);
var
  _LoadedImage : PSDL_Surface;
  _pix : PUInt32;
begin
  _LoadedImage := IMG_Load(PChar(AFile));
  if _LoadedImage <> nil then
    Image := SDL_DisplayFormat(_LoadedImage)
  else
    WriteLn('Blad ladowania: ' + AFile);
  SDL_FreeSurface(_LoadedImage);
  _pix := Image^.pixels;
  FColorKey := _pix^;
  //FColorKey := SDL_MapRGB(Image^.format, 255, 0, 255);
  if AAlpha then
    SDL_SetColorKey(Image, SDL_RLEACCEL or SDL_SRCCOLORKEY, FColorKey);
end;

procedure TGroundImage.Draw;
var
  //roznice w wielkosci image'a i screen'a razy polozenie kursora
  _xsub, _ysub : Integer;
  _offset : TSDL_Rect;
begin
  _xsub := Round((FScreen^.w - Image^.w) * (FMousePercentPointer^.x / 1000));
  _ysub := Round((FScreen^.h - Image^.h) * (FMousePercentPointer^.y / 1000));
  _offset.x := _xsub;
  _offset.y := _ysub;
  
  //blit
  SDL_BlitSurface(Image, nil, FScreen, @_offset);
end;

end.

