unit MiniEngine;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Configuration, sdl, TextManager, Main_Const, Timer,
  GroundImage, sdl_image, Worm, TeamManager;
  
type

  { TMiniEngine }

  TMiniEngine = class
  private
    RysujTest : Boolean; //test
    FScreen : PSDL_Surface;
    //czas turi i gry
    FTourTime,
    FGameTime : Integer;
    //aktualny czas turi i gry
    FActualTourTime,
    FActualGameTime : Integer;
    Fala : PSDL_Surface;
    Fala1Offset,
    Fala2Offset : Integer;
    Ground : TGroundImage;
    BackGround : TGroundImage;
    Cursor : TSprite;
    Worms : array of array of TWorm;
    procedure SetGameTime(ATime : Integer);
    procedure SetTourTime(ATime : Integer);
    function MinutesToSeconds(AMinutes : Integer) : Integer;
    procedure SetupAndStart;
    function GroundInRect(ARect : PSDL_Rect) : Boolean;
    function GroundInPoint(APoint : PPoint) : Boolean;
  public
    robal : TWorm; //test
    OP,
    OPP : PPoint;
    Timer : PTimer;
    TextMngr : TTextManager;
    property TourTime : Integer read FTourTime write SetTourTime;
    property GameTime : Integer read FGameTime write SetGameTime;
    constructor Create(AScreen : PSDL_Surface);
    destructor Destroy;
    procedure LoadGraphics;
    procedure MoveAll;
    procedure DrawAll;
    procedure MouseMove;
    procedure MouseLClick;
    procedure MouseRClick;
    procedure KeyDown(AKey : Integer);
    procedure KeyUp(AKey : Integer);
  end;

implementation

{ TMiniEngine }

procedure TMiniEngine.SetGameTime(ATime: Integer);
begin
  FGameTime := MinutesToSeconds(ATime);
end;

procedure TMiniEngine.SetTourTime(ATime: Integer);
begin
  FTourTime := MinutesToSeconds(ATime);
end;

function TMiniEngine.MinutesToSeconds(AMinutes: Integer): Integer;
begin
  Result := AMinutes * 60;
end;

procedure TMiniEngine.SetupAndStart;
begin
  //Ustawienie czasow poczatkowych
  FActualTourTime := FTourTime;
  FActualGameTime := FGameTime;
  
  
end;

function TMiniEngine.GroundInRect(ARect: PSDL_Rect): Boolean;
var
  i, j : Integer;
  _pix, _pix0 : PUInt32;
  _TransparentColor : UInt32;
  _Breaked : Boolean;
begin
  Result := False;
  _Breaked := False;
  
  _pix := Ground.Image^.pixels;
  _TransparentColor := _pix^;
  
  for i := ARect^.x to ARect^.x + ARect^.w - 1 do
    for j := ARect^.y to ARect^.y + ARect^.h do
    begin
      _pix0 := _pix + j * Ground.Image^.w + i;
      if _TransparentColor <> _pix0^ then
      begin
        Result := True;
        _Breaked := True;
        Break;
      end;
      if _Breaked then
        Break;
    end;
end;

function TMiniEngine.GroundInPoint(APoint: PPoint): Boolean;
var
  _pix, _pix0 : PUInt32;
  _TransparentColor : UInt32;
begin
  //_pix := Ground.Image^.pixels;
  //_TransparentColor := _pix^;
  
  //_pix0 := PUInt32(_pix + APoint^.y * Ground.Image^.w + APoint^.x);
  
  //Result := _pix0^ <> _TransparentColor;
end;

constructor TMiniEngine.Create(AScreen: PSDL_Surface);
begin
  FScreen := AScreen;
  
  //tworzenie obiektow
  TextMngr := TTextManager.Create(FScreen);

end;

destructor TMiniEngine.Destroy;
begin
  //zwolnienie grafik
  SDL_FreeSurface(Fala);
  FreeAndNil(Ground);
  FreeAndNil(BackGround);
  
  inherited;
end;

procedure TMiniEngine.LoadGraphics;
var
  _TempImage : PSDL_Surface;
  _ColorKey : UInt32;
  i, j : Integer;
  _TempRect : TSDL_Rect;
begin
  //zaladowanie czcionki
  TextMngr.LoadFont(MainConfig.MainDir + '/FreeSans.ttf', FONT_SIZE_NORMAL);
  TextMngr.FontColor.r := 199;
  TextMngr.FontColor.g := 199;
  TextMngr.FontColor.b := 199;
  
  //fala
  _TempImage := IMG_Load(PChar(MainConfig.MainDir + '/gfx/fala.png'));
  Fala := SDL_DisplayFormat(_TempImage);
  SDL_FreeSurface(_TempImage);
  _ColorKey := SDL_MapRGB(Fala^.format, 255, 0, 255);
  SDL_SetColorKey(Fala, SDL_RLEACCEL or SDL_SRCCOLORKEY, _ColorKey);
  
  //grunt
  Ground := TGroundImage.Create(FScreen, OPP);
  Ground.LoadImage(MainConfig.MainDir + '/mapy/map' + IntToStr(MainConfig.Map+1) + '.png', True);
  
  //tlo
  BackGround := TGroundImage.Create(FScreen, OPP);
  BackGround.LoadImage(MainConfig.MainDir + '/tla/tlo' + IntToStr(MainConfig.BackGround+1) + '.jpg');
  
  //test
  robal := TWorm.Create(OP, OPP, FScreen, Ground.Image);
  robal.Graf_Stand := TSprite.Create(FScreen);
  robal.Graf_Stand.LoadGraf(MainConfig.MainDir + '/gfx/stan1.png');
  robal.Graf_Stand.Col := 11;
  robal.Graf_Stand.Row := 1;
  robal.Graf_Stand.Animation := True;
  robal.Graf_Stand.AnimSpeed := 3;
  robal.Graf_Stand.Loop := True;
  robal.Graf_Stand.Frame := 1;
  robal.Graf_Stand.Offset.x := 100;
  robal.Graf_Stand.Offset.y := 100;
  robal.Graf_Current := @robal.Graf_Stand;
  
  //kursor (tylko Win?)
  Cursor := TSprite.Create(FScreen);
  Cursor.LoadGraf(MainConfig.MainDir + '/gfx/arrow2.png');
  Cursor.Col := 1;
  Cursor.Row := 1;
  
  //robale
  Randomize;
  SetLength(Worms, TeamMngrInGame.TeamCount);
  for i := 0 to High(Worms) do
  begin
    SetLength(Worms[i], TeamMngrInGame.GetTeam(i).MemberCount);
    for j := 0 to High(Worms[i]) do
    begin
      Worms[i][j] := TWorm.Create(OP, OPP, FScreen, Ground.Image);
      with Worms[i][j] do
      begin
        Graf_Stand := TSprite.Create(FScreen);
        Graf_Stand.LoadGraf(MainConfig.MainDir + '/gfx/worm' + IntToStr(TeamMngrInGame.GetTeam(i).Nation + 1) + '_right.png');
        Graf_Stand.Row := 1;
        Graf_Stand.Col := 7;
        Graf_Stand.Frame := 1;

        Graf_MoveLeft := TSprite.Create(FScreen);
        Graf_MoveLeft.LoadGraf(MainConfig.MainDir + '/gfx/worm' + IntToStr(TeamMngrInGame.GetTeam(i).Nation + 1) + '_left.png');
        Graf_MoveLeft.Row := 1;
        Graf_MoveLeft.Col := 7;
        Graf_MoveLeft.Animation := True;
        Graf_MoveLeft.Loop := True;
        Graf_MoveLeft.AnimSpeed := 3;
        
        Graf_MoveRight := TSprite.Create(FScreen);
        Graf_MoveRight.LoadGraf(MainConfig.MainDir + '/gfx/worm' + IntToStr(TeamMngrInGame.GetTeam(i).Nation + 1) + '_right.png');
        Graf_MoveRight.Row := 1;
        Graf_MoveRight.Col := 7;
        Graf_MoveRight.Animation := True;
        Graf_MoveRight.Loop := True;
        Graf_MoveRight.AnimSpeed := 3;
        
        Graf_JumpLeft := TSprite.Create(FScreen);
        Graf_JumpLeft.LoadGraf(MainConfig.MainDir + '/gfx/worm' + IntToStr(TeamMngrInGame.GetTeam(i).Nation + 1) + '_jump_right.png');

        Graf_JumpRight := TSprite.Create(FScreen);
        Graf_JumpRight.LoadGraf(MainConfig.MainDir + '/gfx/worm' + IntToStr(TeamMngrInGame.GetTeam(i).Nation + 1) + '_jump_right.png');

        Graf_Current := @Graf_Stand;
        
        //pasek stanu
        State := TSprite.Create(FScreen);
        State.LoadGraf(MainConfig.MainDir + '/gfx/stan1.png');
        State.Col := 11;
        State.Row := 1;
        State.Animation := False;
        State.Loop := False;
        
        //ustawienie losowe pozycji robakow
        _TempRect.x := Position.x;
        _TempRect.y := Position.y;
        _TempRect.w := Graf_Current^.Graf^.w;
        _TempRect.h := Graf_Current^.Graf^.h;
        Position.x := Random(Ground.Image^.w - _TempRect.w - 10);
        Position.y := Random(Ground.Image^.h - _TempRect.h - 100);
        Move;
        while GroundInRect(@_TempRect) do
        begin
          Position.x := Random(Ground.Image^.w - _TempRect.w - 10);
          Position.y := Random(Ground.Image^.h - _TempRect.h - 100);
          Move;
          _TempRect.x := Position.x;
          _TempRect.y := Position.y;
        end;
        while not GroundInRect(@_TempRect) do
        begin
          Position.y := Position.y + 1;
          Move;
          _TempRect.y := Position.y;
          if Position.y > Ground.Image^.h - _TempRect.h - 10 then
            Break;
        end;
      end;
    end;
  end;
  
  //rozpoczecie rozgrywki
  SetupAndStart;
end;

procedure TMiniEngine.MoveAll;
var
  i, j : Integer;
begin
  //fale
  Fala1Offset := Fala1Offset + 2;
  Fala2Offset := Fala2Offset + 1;
  if Fala1Offset > Fala^.w - 1 then
    Fala1Offset := 0;
  if Fala2Offset > Fala^.w - 1 then
    Fala2Offset := 0;
  
  
  //test
  robal.Move;
  
  //robaki
  for i := 0 to High(Worms) do
  begin
    for j := 0 to High(Worms[i]) do
    begin
      Worms[i][j].Move;
    end;
  end;
  
  //kursor (tylko Win?)
  Cursor.Offset.x := OP^.x;
  Cursor.Offset.y := OP^.y;
  Cursor.Move;
end;

procedure TMiniEngine.DrawAll;
var
  i : Integer;
  _offset,
  _offset2 : TSDL_Rect;
  j: Integer;
begin
  //tlo
  BackGround.Draw;
  
  //tylnia fala
  _offset.x := -Fala^.w;
  _offset.y := Round((FScreen^.h - Ground.Image^.h) * (OPP^.y / 1000)) + Ground.Image^.h - Fala^.h - Fala^.h div 2;
  _offset2.x := _offset.x + Fala2Offset;
  _offset2.y := _offset.y;
  for i := 0 to FScreen^.w div Fala^.w do
  begin
    SDL_BlitSurface(Fala, nil, FScreen, @_offset2);
    _offset.x := _offset.x + Fala^.w;
    _offset2.x := _offset.x + Fala2Offset - Fala^.w;
  end;
  SDL_BlitSurface(Fala, nil, FScreen, @_offset2);

  //grunt
  Ground.Draw;
  
  //robaki
  for i := 0 to High(Worms) do
  begin
    for j := 0 to High(Worms[i]) do
    begin
      Worms[i][j].Draw;
    end;
  end;

  //przednia fala
  _offset.x := -Fala^.w;
  _offset.y := Round((FScreen^.h - Ground.Image^.h) * (OPP^.y / 1000)) + Ground.Image^.h - Fala^.h;
  _offset2.x := _offset.x + Fala1Offset;
  _offset2.y := _offset.y;
  for i := 0 to FScreen^.w div Fala^.w do
  begin
    SDL_BlitSurface(Fala, nil, FScreen, @_offset2);
    _offset.x := _offset.x + Fala^.w;
    _offset2.x := _offset.x + Fala1Offset - Fala^.w;
  end;
  SDL_BlitSurface(Fala, nil, FScreen, @_offset2);
  
  //test
  robal.Draw;

  //test wyswietlania tekstu (FPS)
  TextMngr.x := 0;
  TextMngr.y := 0;
  TextMngr.Align := ALIGN_LEFT;
  TextMngr.DrawText(IntToStr(Timer^.Fps));
  
  TextMngr.x := FScreen^.w div 2;
  TextMngr.y := 5;
  TextMngr.Align := ALIGN_CENTER;
  TextMngr.DrawText('Czas gry: ' + IntToStr(FActualGameTime) + 's');
  
  TextMngr.x := FScreen^.w div 2;
  TextMngr.y := 25;
  TextMngr.Align := ALIGN_CENTER;
  TextMngr.DrawText('Czas tury: ' + IntToStr(FActualTourTime) + 's');

  //test
  if RysujTest then
  begin
    TextMngr.x := FScreen^.w div 2;
    TextMngr.y := 60;
    TextMngr.Align := ALIGN_CENTER;
    TextMngr.DrawText('GRUNT ' + IntToStr(OP^.x) + ' ' + IntToStr(OP^.y));
  end;
  
  //kursor (tylko Win?)
  {$ifdef Win32}
  Cursor.Draw;
  {$endif}
end;

procedure TMiniEngine.MouseMove;
var
  _TempRect : TSDL_Rect;
  _TempPoint : TPoint;
begin
  //test
  _TempRect.x := OP^.x + Round((FScreen^.w - Ground.Image^.w) * (OPP^.x / 1000));
  _TempRect.y := OP^.y + Round((FScreen^.h - Ground.Image^.h) * (OPP^.y / 1000));
  _TempRect.w := 2;
  _TempRect.h := 2;
  _TempPoint.x := 10;
  _TempPoint.y := 10;

  if GroundInPoint(@_TempPoint) then
  begin
    RysujTest := True;
  end;
end;

procedure TMiniEngine.MouseLClick;
begin

end;

procedure TMiniEngine.MouseRClick;
begin

end;

procedure TMiniEngine.KeyDown(AKey: Integer);
begin

end;

procedure TMiniEngine.KeyUp(AKey: Integer);
begin

end;

end.

