unit TeamManager;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils; 
  
type

  TArrayOfString = array of string;

  PTeam = ^TTeam;

  { TTeam }

  TTeam = class
  private
    FName : string;
    FNation : Integer;
    FMembers : TArrayOfString;
  public
    ActiveMember : Integer;
    property Name : string read FName write FName;
    property Nation : Integer read FNation write FNation;
    property Members : TArrayOfString read FMembers write FMembers;
    
    procedure AddMember(const AName : String);
    function MemberCount : Integer;
  end;
  
  
  PTeamMngr = ^TTeamMngr;

  { TTeamMngr }

  TTeamMngr = class
  private
    FTeams : array of TTeam;
  public
    ActiveTeam : Integer;
    procedure AddTeam(const ATeam : TTeam);
    function TeamCount : Integer;
    function GetTeam(AIndex : Integer) : TTeam;
  end;
  

var
  TeamMngr,
  TeamMngrInGame : TTeamMngr;
  
implementation

{ TTeamMngr }

procedure TTeamMngr.AddTeam(const ATeam: TTeam);
begin
  SetLength(FTeams, High(FTeams) + 2);
  FTeams[High(FTeams)] := ATeam;
end;

function TTeamMngr.TeamCount: Integer;
begin
  Result := High(FTeams) + 1;
end;

function TTeamMngr.GetTeam(AIndex: Integer): TTeam;
begin
  if AIndex <= High(FTeams) then
    Result := FTeams[AIndex]
  else
    Result := nil;
end;

{ TTeam }

procedure TTeam.AddMember(const AName: String);
begin
  SetLength(FMembers, High(FMembers) + 2);
  FMembers[High(FMembers)] := AName;
end;

function TTeam.MemberCount: Integer;
begin
  if FMembers <> nil then
    Result := High(FMembers) + 1
  else
    Result := -1;
end;

initialization
  TeamMngr := TTeamMngr.Create;
  TeamMngrInGame := TTeamMngr.Create;
  
finalization
  FreeAndNil(TeamMngr);
  FreeAndNil(TeamMngrInGame);
  
end.

