unit TextManager;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sdl_ttf, sdl;
  
type

  { TTextManager }

  TTextManager = class
  private
    FOffset : TSDL_Rect;
    FFont : PTTF_Font;
    FFontColor : TSDL_Color;
    FFontSurface : PSDL_Surface;
    FScreen : PSDL_Surface;
    FAlign : Integer;
    function GetX : Integer;
    function GetY : Integer;
    procedure SetX(AX : Integer);
    procedure SetY(AY : Integer);
  public
    property x : Integer read GetX write SetX;
    property y : Integer read GetY write SetY;
    property Offset : TSDL_Rect read FOffset write FOffset;
    property FontColor : TSDL_Color read FFontColor write FFontColor;
    property Align : Integer read FAlign write FAlign;
    
    constructor Create(AScreen : PSDL_Surface);
    procedure LoadFont(AFile : String; ASize : Integer);
    procedure DrawText(AText : String);
  end;
  
const
  ALIGN_LEFT = 1;
  ALIGN_CENTER = 2;
  ALIGN_RIGHT = 3;


implementation

{ TTextManager }

function TTextManager.GetX: Integer;
begin
  Result := FOffset.x;
end;

function TTextManager.GetY: Integer;
begin
  Result := FOffset.y;
end;

procedure TTextManager.SetX(AX: Integer);
begin
  FOffset.x := AX;
end;

procedure TTextManager.SetY(AY: Integer);
begin
  FOffset.y := AY;
end;

constructor TTextManager.Create(AScreen: PSDL_Surface);
begin
  FScreen := AScreen;
  FAlign := ALIGN_LEFT;
  
  if TTF_Init < 0 then
    WriteLn('Blad przy inicjacji czcionek');
end;

procedure TTextManager.LoadFont(AFile: String; ASize: Integer);
begin
  FFont := TTF_OpenFont(PChar(AFile), ASize);
end;

procedure TTextManager.DrawText(AText: String);
var
  _TempOffset : TSDL_Rect;
begin
  FFontSurface := TTF_RenderText_Solid(FFont, PChar(AText), FFontColor);
  
  case FAlign of
  ALIGN_LEFT:
    _TempOffset.x := 0;
  ALIGN_CENTER:
    _TempOffset.x := -FFontSurface^.w div 2;
  ALIGN_RIGHT:
    _TempOffset.x := -FFontSurface^.w;
  end;
    
  _TempOffset.x := _TempOffset.x + FOffset.x;
  _TempOffset.y := FOffset.y;
    
  SDL_BlitSurface(FFontSurface, nil, FScreen, @_TempOffset);
  SDL_FreeSurface(FFontSurface);
end;

end.

