unit Worm;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sdl, sdl_image;
  
type

  { TSprite }

  PSprite = ^TSprite;
  TSprite = class
  private

    FClip : array of TSDL_Rect;
    FCol,
    FRow : Word;
    FClipCount : Word;
    FLoop : Boolean;
    FAnimSpeed : Word;
    FAnimation : Boolean;
    FAnimDirection : Byte;
    FFrame : Integer;
    FGlobalFrameCounter : Cardinal;
    FDrawScreen : PSDL_Surface;
    FOffset : TSDL_Rect;
    procedure SetRow(ARow : Word);
    procedure SetCol(ACol : Word);
    procedure UpdateClips;
  public
    Graf : PSDL_Surface;

    property Row : Word read FRow write SetRow;
    property Col : Word read FCol write SetCol;
    
    //czy powtarzac animacje
    property Loop : Boolean read FLoop write FLoop;
    //predkosc animacji - przez ile klatek wyswietlac jeden Clip
    property AnimSpeed : Word read FAnimSpeed write FAnimSpeed;
    //czy animowac - jesli false to wyswietla ustawiony clip
    property Animation : Boolean read FAnimation write FAnimation;
    //kierunek animacji
    property AnimDirection : Byte read FAnimDirection write FAnimDirection;
    //aktualny Clip (klatka)
    property Frame : Integer read FFrame write FFrame;
    //przesuniecie
    property Offset : TSDL_Rect read FOffset write FOffset;
    
    constructor Create(ADrawScreen : PSDL_Surface);
    destructor Destroy;
    //rysuj
    procedure Draw;
    //przesun do nastepnego Clip'u
    procedure Move;
    //zaladowanie grafiki
    procedure LoadGraf(const AFile : String);
  end;

  { TWorm }

  TWorm = class
  private
    //wskaznik na punkt odniesienia
    FOP : PPoint;
    FOPP : PPoint;
    FScreen : PSDL_Surface;
    FGround : PSDL_Surface;
    FEnergy : Integer;
    procedure SetEnergy(AEnergy : Integer);
    function GetEnergy : Integer;
  public
    //pozycja wzgledna
    Position : TPoint;
    
    //sprite'y
    Graf_MoveLeft,
    Graf_MoveRight,
    Graf_JumpLeft,
    Graf_JumpRight,
    Graf_Stand : TSprite;
    
    Graf_Current : PSprite;
    
    State : TSprite;
    
    property Energy : Integer read GetEnergy write SetEnergy;
    
    constructor Create(AOP, AOPP : PPoint; AScreen, AGround : PSDL_Surface);
    procedure Move;
    procedure Draw;
  end;
  
const
  ANIM_FORWARD = 1;
  ANIM_BACKWARD = 2;

implementation

{ TSprite }

procedure TSprite.SetRow(ARow: Word);
begin
  FRow := ARow;
  UpdateClips;
end;

procedure TSprite.SetCol(ACol: Word);
begin
  FCol := ACol;
  UpdateClips;
end;

procedure TSprite.UpdateClips;
var
  j: Integer;
  i: Integer;
begin
  FClipCount := FRow * FCol;
  SetLength(FClip, FClipCount);
  for i := 0 to FRow - 1 do
  begin
    for j := 0 to FCol - 1 do
    begin
      with FClip[i*FRow+j] do
      begin
        w := Graf^.w div FCol;
        h := Graf^.h div FRow;
        x := w * j;
        y := h * i;
      end;
    end;
  end;
end;

constructor TSprite.Create(ADrawScreen : PSDL_Surface);
begin
  FFrame := 0;
  FAnimation := False;
  FLoop := False;
  FAnimSpeed := 1;
  FDrawScreen := ADrawScreen;
  FAnimDirection := ANIM_FORWARD;
end;

destructor TSprite.Destroy;
begin
  SDL_FreeSurface(Graf);
  inherited;
end;

procedure TSprite.Draw;
begin
  SDL_BlitSurface(Graf, @FClip[FFrame], FDrawScreen, @FOffset);
end;

procedure TSprite.Move;
begin
  //jesli animowany
  if FAnimation and (FGlobalFrameCounter mod FAnimSpeed = 0) then
  begin

    if FAnimDirection = ANIM_FORWARD then
      //jesli animacja w przod
      Inc(FFrame)
    else
      //jesli animacja w tyl
      Dec(FFrame);
      
    if (FFrame > FClipCount - 1) and (FAnimDirection = ANIM_FORWARD) then
      if FLoop then
        FFrame := 0
      else
        FFrame := FClipCount - 1;
        
    if (FFrame < 0) and (FAnimDirection = ANIM_BACKWARD) then
      if FLoop then
        FFrame := FClipCount - 1
      else
        FFrame := 0;
  end;
  //zwiekszamy globalny licznik klatek
  FGlobalFrameCounter := FGlobalFrameCounter + 1;
end;

procedure TSprite.LoadGraf(const AFile: String);
var
  _LoadedImage : PSDL_Surface;
  _pix : PUInt32;
  _ColorKey : UInt32;
begin
  _LoadedImage := IMG_Load(PChar(AFile));
  if _LoadedImage <> nil then
  begin
    Graf := SDL_DisplayFormat(_LoadedImage);
    SDL_FreeSurface(_LoadedImage);
  end else
    WriteLn('Nie zaladowano spritea: ' + AFile);
    
  _pix := Graf^.pixels;
  _ColorKey := _pix^;
  SDL_SetColorKey(Graf, SDL_RLEACCEL or SDL_SRCCOLORKEY, _ColorKey);
end;

{ TWorm }

procedure TWorm.SetEnergy(AEnergy: Integer);
begin
  FEnergy := AEnergy div 11;
end;

function TWorm.GetEnergy: Integer;
begin
  Result := FEnergy * 11;
end;

constructor TWorm.Create(AOP, AOPP : PPoint; AScreen, AGround : PSDL_Surface);
begin
  FOP := AOP;
  FOPP := AOPP;
  FScreen := AScreen;
  FGround := AGround;
  FEnergy := 10;
end;

procedure TWorm.Move;
begin
  Graf_Current^.Move;
  Graf_Current^.Offset.x := Position.x + Round((FScreen^.w - FGround^.w) * (FOPP^.x / 1000));
  Graf_Current^.Offset.y := Position.y + Round((FScreen^.h - FGround^.h) * (FOPP^.y / 1000));
  
  //pasek stanu
  if State <> nil then
  begin
    State.Offset.x := Graf_Current^.Offset.x - 5;
    State.Offset.y := Graf_Current^.Offset.y - 25;
    State.Frame := FEnergy;
  end;
end;

procedure TWorm.Draw;
begin
  Graf_Current^.Draw;
  
  //pasek stanu
  if State <> nil then
  begin
    State.Draw;
  end;
end;

end.

